#!/usr/bin/perl

use strict;
use warnings;

use DBI;
use Getopt::Std;
use Term::ReadKey;
use HTML::Diff;

# Process command line arguments
my %opts;
getopts('sul:p:', \%opts);
my $version_from = shift or die "Usage: $0 [-su] [-l <lang>] [-p <mysql_password>] <version_from> <version_to>\n";
my $version_to = shift or die "Usage: $0 [-su] [-l <lang>] [-p <mysql_password>] <version_from> <version_to>\n";

my $lang = $opts{l} || 'en';

# Connect to database
my ($data_source, $db_user);
if ($opts{s}) {
  # ssh -L 6666:127.0.0.1:3306 gibus@pi.serverside.fr
	$data_source = 'dbi:mysql:dbname=pi_tpdb;host=127.0.0.1;port=6666';
	$db_user = 'pi_tpdb';
}
else {
	$data_source = 'dbi:mysql:dbname=tpdb';
	$db_user = 'gibus';
}

# Get mysql password
my $db_passwd;
if (defined $opts{p}) {
    $db_passwd = $opts{p};
}
else {
    print "Enter mysql passwd for $db_user:\n";
    ReadMode 'noecho';
    $db_passwd = ReadLine 0;
    chomp $db_passwd;
    ReadMode 'normal';
}

my $dbh = DBI->connect($data_source, $db_user, $db_passwd) or die $DBI::errstr;

my $limit = '';
# Select only COD/2007/0247 when going from ITRE report
# except when going to IMCO report
if ($version_from eq 'g' and $version_to ne 'h') {
    $limit .= " AND amending=1";
}
# Select only COD/2007/0248 when going from IMCO report
# except when going to ITRE report
if ($version_from eq 'h' and $version_to ne 'g') {
    $limit .= " AND amending=2";
}
# Select only COD/2007/0247 when going to ITRE report
# except when going from original directives or Commission initial proposal
if ($version_to eq 'g' and $version_from ne 'a' and $version_from ne 'b') {
    $limit .= " AND amending=1";
}
# Select only COD/2007/0248 when going to IMCO report
# except when going from original directives or Commission initial proposal
if ($version_to eq 'h' and $version_from ne 'a' and $version_from ne 'b') {
    $limit .= " AND amending=2";
}

# Get 'from' chunks
my $sql_from = "SELECT * FROM tp_text WHERE version='$version_from' AND lang='$lang'$limit";
my $sth_from = $dbh->prepare($sql_from);
$sth_from->execute;
my $from = $sth_from->fetchall_hashref( [ qw(amended number) ] );

# Get 'to' chunks
my $sql_to = "SELECT * FROM tp_text WHERE version='$version_to' AND lang='$lang'$limit";
my $sth_to = $dbh->prepare($sql_to);
$sth_to->execute;
my $to = $sth_to->fetchall_hashref( [ qw(amended number) ] );

# Compute diff chunks
my %diff;
# Iterate through 'to' chunks
foreach my $amended (keys %{$to}) {
    foreach my $number (keys %{$to->{$amended}}) {
        my $to_chunk = $to->{$amended}->{$number}->{text};
        # Find related 'from' chunk if any
        my $from_chunk;
        my $num;
        if (exists $from->{$amended}) {
            # 'from' chunk with same number
            if (exists $from->{$amended}->{$number}) {
                $from_chunk = $from->{$amended}->{$number}->{text};
                $num = $number;
                $diff{$amended}->{$num}->{number_from} = $number;
            }
            # 'from' chunk with added paragraphs on the same level
            elsif (exists $from->{$amended}->{$number.'1'}) {
                $from_chunk = $from->{$amended}->{$number.'1'}->{text};
                $num = $number.'1';
                $diff{$amended}->{$num}->{number_from} = $number.'1';
            }
            # 'from' chunk with added paragraphs on the previous level
            elsif (exists $from->{$amended}->{substr($number, 0, length($number)-1).'1'.substr($number, -1, 1)}) {
                $from_chunk = $from->{$amended}->{substr($number, 0, length($number)-1).'1'.substr($number, -1, 1)}->{text};
                $num = substr($number, 0, length($number)-1).'1'.substr($number, -1, 1);
                $diff{$amended}->{$num}->{number_from} = substr($number, 0, length($number)-1).'1'.substr($number, -1, 1);
            }
            # 'from' chunk with only one paragraphs on the same level
            elsif (substr($number, -1, 1) eq '1' and exists $from->{$amended}->{substr($number, 0, length($number)-1)}) {
                $from_chunk = $from->{$amended}->{substr($number, 0, length($number)-1)}->{text};
                $num = $number;
                $diff{$amended}->{$num}->{number_from} = substr($number, 0, length($number)-1);
            }
            # 'from' chunk with only one paragraphs on the previous level
            elsif (substr($number, -2, 1) eq '1' and exists $from->{$amended}->{substr($number, 0, length($number)-2).substr($number, length($number)-1,1)}) {
                $from_chunk = $from->{$amended}->{substr($number, 0, length($number)-2).substr($number, length($number)-1,1)}->{text};
                $num = $number;
                $diff{$amended}->{$num}->{number_from} = substr($number, 0, length($number)-2).substr($number, length($number)-1,1);
            }
        }

        # No 'from' chunk, 'to' chunk is new
        unless (defined $from_chunk) {
            $from_chunk = '';
            $num = $number;
            $diff{$amended}->{$num}->{number_from} = undef;
        }

        # Compute diff
        $diff{$amended}->{$num}->{diff} = diff($from_chunk, $to_chunk);
        delete $from->{$amended}->{$diff{$amended}->{$num}->{number_from}} if $from_chunk;

        # Build result
        $diff{$amended}->{$num}->{version_from} = $version_from;
        $diff{$amended}->{$num}->{version_to} = $version_to;
        $diff{$amended}->{$num}->{nature} = $to->{$amended}->{$number}->{nature};
        $diff{$amended}->{$num}->{amending} = $to->{$amended}->{$number}->{amending};
        $diff{$amended}->{$num}->{amended} = $to->{$amended}->{$number}->{amended};
        $diff{$amended}->{$num}->{number_to} = $number;
        $diff{$amended}->{$num}->{lang} = $to->{$amended}->{$number}->{lang};
    }
}

# Iterate through 'from' chunks that have been deleted
foreach my $amended (keys %{$from}) {
    foreach my $number (keys %{$from->{$amended}}) {
        $diff{$amended}->{$number}->{version_from} = $version_from;
        $diff{$amended}->{$number}->{version_to} = $version_to;
        $diff{$amended}->{$number}->{nature} = $from->{$amended}->{$number}->{nature};
        $diff{$amended}->{$number}->{amending} = $from->{$amended}->{$number}->{amending};
        $diff{$amended}->{$number}->{amended} = $from->{$amended}->{$number}->{amended};
        $diff{$amended}->{$number}->{number_to} = undef;
        $diff{$amended}->{$number}->{number_from} = $number;
        $diff{$amended}->{$number}->{lang} = $from->{$amended}->{$number}->{lang};
        $diff{$amended}->{$number}->{diff} = "<s>$from->{$amended}->{$number}->{text}</s>";

        unless ($diff{$amended}->{$number}->{amending}) {
            $diff{$amended}->{$number}->{amending} = $diff{$amended}->{$number}->{amended} < 4 ? 1 : 2;
        }
    }
}

# Insert diff in mysql
my ($ins, $upd, $err) = (0, 0, 0);
foreach my $amended (keys %diff) {
    foreach my $number (keys %{$diff{$amended}}) {
        # Check if text is already known in database
        my $sql_check = "SELECT version_from FROM tp_diff_texts WHERE version_from='$version_from' AND version_to='$version_to' AND nature='$diff{$amended}->{$number}->{nature}' AND amending='$diff{$amended}->{$number}->{amending}' AND amended='$diff{$amended}->{$number}->{amended}' AND number='$number' AND lang='$lang'";
        (my $check) = $dbh->selectrow_array($sql_check);
        unless ($check) {
            my $sql_insert = "INSERT INTO tp_diff_texts VALUES ('$diff{$amended}->{$number}->{version_from}','$diff{$amended}->{$number}->{version_to}','$diff{$amended}->{$number}->{nature}','$diff{$amended}->{$number}->{amending}','$diff{$amended}->{$number}->{amended}','$number',".($diff{$amended}->{$number}->{number_from} ? "'$diff{$amended}->{$number}->{number_from}'" : "NULL").",".($diff{$amended}->{$number}->{number_to} ? "'$diff{$amended}->{$number}->{number_to}'" : "NULL").",'$lang',".$dbh->quote($diff{$amended}->{$number}->{diff}).")";
            $dbh->do($sql_insert);
            warn "$diff{$amended}->{$number}->{version_from} $diff{$amended}->{$number}->{version_to} $diff{$amended}->{$number}->{nature} $diff{$amended}->{$number}->{amending} $diff{$amended}->{$number}->{amended} $number $lang inserted\n";
            $ins++;
        }
        elsif ($opts{u}) {
            my $sql_update = "UPDATE tp_diff_texts SET diff=".$dbh->quote($diff{$amended}->{$number}->{diff})." WHERE version_from='$version_from' AND version_to='$version_to' AND nature='$diff{$amended}->{$number}->{nature}' AND amending='$diff{$amended}->{$number}->{amending}' AND amended='$diff{$amended}->{$number}->{amended}' AND number='$number' AND lang='$lang'";
            $dbh->do($sql_update);
            warn "$diff{$amended}->{$number}->{version_from} $diff{$amended}->{$number}->{version_to} $diff{$amended}->{$number}->{nature} $diff{$amended}->{$number}->{amending} $diff{$amended}->{$number}->{amended} $number $lang updated\n";
            $upd++;
        }
        else {
  	        warn "Error: cannot insert $diff{$amended}->{$number}->{version_from} $diff{$amended}->{$number}->{version_to} $diff{$amended}->{$number}->{nature} $diff{$amended}->{$number}->{amending} $diff{$amended}->{$number}->{amended} $number $lang\n";
            $err++;
        }
    }
}
print "Inserted: $ins, updated: $upd, errors: $err\n";

# Compute diff
sub diff {
    my $from = shift;
    my $to = shift;
    my @diff = @{html_word_diff($from, $to)};
    my $diff = '';
    foreach my $chunk (@diff) {
        # Unchanged
        if ($chunk->[0] eq 'u') {
            $diff .= $chunk->[1];
        }
        # Deleted
        elsif ($chunk->[0] eq '-') {
            $diff .= "<s>".$chunk->[1]."</s>";
        }
        # Added or updated
        else {
            $diff .= "'''''".$chunk->[2]."'''''";
        }
    }
    return $diff;
}
