#!/usr/bin/perl

use strict;
use warnings;

use FindBin qw($Bin);
use lib "$Bin";
(my $root_dir = $Bin) =~ s/\/[^\/]*$//o;

use DBI;
use Getopt::Std;
use Term::ReadKey;
use HTML::Diff;
use File::Path;

# Process command line arguments
my %opts;
getopts('sl:a:b:n:p:', \%opts);
my $version = shift or die "Usage: $0 [-s] [-l <lang>] [-a <amended_directive>] [-b <amending_directive>] [-n <article_number>] [-p <mysql_password>] <version_text>\n";

# Lang
my $lang = defined($opts{l}) ? $opts{l} : 'en';
my $where_clause = " version='$version' AND lang='$lang'";

# Amended
if (defined $opts{a}) {
    die "Amended directive should be:\n* 1 for Access Directive;\n* 2 for Authorisation directive;\n* 3 for Framework Directive;\n* 4 for Universal Service Directive; or\n* 5 for ePrivacy Directive.\n"
        if ($opts{a}<1 or $opts{a}>5);
    $where_clause .= " AND amended='$opts{a}'";
}

# Amending
if (defined $opts{b}) {
    die "Amending directive should be:\n* 1 for COD/2007/0247 (Access, Authorisation and Framework); or\n* 2 for COD/2007/0248 (Universal Service and ePrivacy).\n"
        if ($opts{b}<0 or $opts{b}>2);
    if (defined $opts{a}) {
        die "Amending directive and amended directive are not consistent, amending directive should be:\n* 1 for COD/2007/0247 (Access, Authorisation and Framework); or\n* 2 for COD/2007/0248 (Universal Service and ePrivacy).\n"
        if (($opts{b}==1 and $opts{a}>3) or ($opts{b}==2 and $opts{a}<4));
    }
    $where_clause .= " AND amending='$opts{b}'";
}

# Number
if (defined $opts{n}) {
    die "Article (or recital) number should begin by two (2) digits.\n"
        if ($opts{n} and $opts{n}!~/^\d\d/o);
    die "Amended directive should be set on command line arguments to also set article (or recital, or annex) number.\n"
        unless defined $opts{a};
    $where_clause .= " AND number LIKE '$opts{n}%'";
    $where_clause .= " AND not(number LIKE '$opts{n}\@%'" unless $opts{n} =~ /\@/o;
}

# Connect to database
my ($data_source, $db_user);
if (defined $opts{s}) {
    # ssh -L 6666:127.0.0.1:3306 gibus@pi.serverside.fr
	$data_source = 'dbi:mysql:dbname=pi_tpdb;host=127.0.0.1;port=6666;mysql_enable_utf8=1';
	$db_user = 'pi_tpdb';
}
else {
	$data_source = 'dbi:mysql:dbname=tpdb';
	$db_user = 'gibus';
}

# Get mysql password
my $db_passwd;
if (defined $opts{p}) {
    $db_passwd = $opts{p};
}
else {
    print "Enter mysql passwd for $db_user:\n";
    ReadMode 'noecho';
    $db_passwd = ReadLine 0;
    chomp $db_passwd;
    ReadMode 'normal';
}

my $dbh = DBI->connect($data_source, $db_user, $db_passwd) or die $DBI::errstr;

# Get page wiki name
my $wiki_name = "Telecoms_Package_";

if ($opts{a}) {
    my $sql_amended_wiki = "SELECT short_name FROM tp_amended WHERE id='$opts{a}' and lang='en'";
    (my $short_name) = $dbh->selectrow_array($sql_amended_wiki);
    $short_name =~ s/ /_/go;
    $wiki_name .= $short_name;

    if ($opts{n}) {
        $opts{n} =~ s/^0//o;
        if ($opts{n} =~ s/^A(\d)//o) {
            my $roman = $1;
            $roman =~ s/^1$/I/o;
            $roman =~ s/^2$/II/o;
            $roman =~ s/^3$/III/o;
            $roman =~ s/^4$/IV/o;
            $roman =~ s/^5$/V/o;
            $wiki_name .= "_Annexe_$roman";
        }
        elsif ($opts{n} =~ s/^[^:]+://o) {
            $wiki_name .= '_Recital';
        }
        else {
            $wiki_name .= '_Article';
        }
        $opts{n} =~ s/(.)/_$1/go;
        $opts{n} =~ s/_\@_(.)/\l$1/go;
        $wiki_name .= $opts{n};
    }
}
elsif ($opts{b}) {
    my $sql_amending_wiki = "SELECT number FROM tp_amending WHERE id='$opts{b}' and lang='en'";
    (my $number) = $dbh->selectrow_array($sql_amending_wiki);
    $number =~ s/\//_/go;
    $wiki_name .= $number;
}
else {
    $wiki_name =~ s/_$//o;
}

my $sql_version_wiki = "SELECT origin, description FROM tp_versions WHERE id='$version' and lang='en'";
my ($origin_wiki, $description_wiki) = $dbh->selectrow_array($sql_version_wiki);
$wiki_name .= "_$origin_wiki" unless $origin_wiki eq 'Parliament & Council';
if ($description_wiki =~ /&/o) {
    if ((defined($opts{b}) and $opts{b} == 1) or (defined($opts{a}) and $opts{a} < 4)) {
        $description_wiki =~ s/ &.+$//o;
    }
    elsif (not(defined($opts{b})) and not(defined($opts{a}))) {
        $description_wiki =~ s/ & /_/o;
    }
    else {
        $description_wiki =~ s/^.+& //o;
    }
}
$wiki_name .= "_$description_wiki";
$wiki_name =~ s/ /_/go;

# Get langs
my $sql_lang = "SELECT id, name FROM tp_lang";
my $langs  = $dbh->selectall_arrayref($sql_lang);

# Start wiki page
my $wiki_page = "__NOEDITSECTION__\n__FORCETOC__\n<div class=\"LanguageLinks\"><table width=\"100%\"><tr valign=\"top\" style=\"background: #EEF3E2\"><td style=\"width: 25px; padding-left: 0.5em;\">[[Image:Geographylogo.png|25px|Languages]]</td><td style=\"padding: 1px 1em 0; background: #F6F9ED;\">";
foreach my $lang (@$langs) {
    $wiki_page .= "'''[[$wiki_name".($lang->[0] eq 'en' ? '' : "/$lang->[0]")."|$lang->[1]]]''' - ";
}
$wiki_page =~ s/ - $//;
$wiki_page .= "</td></tr></table></div>\n\n";

# Get some title info
my $sql_version = "SELECT origin, description, date FROM tp_versions WHERE id='$version' and lang='$lang'";
my ($origin, $description, $date) = $dbh->selectrow_array($sql_version);
my $name;
if ($origin ne 'Parliament & Council') {
    my $sql_origin = "SELECT name FROM tp_origin WHERE short_name='$origin' and lang='$lang'";
    ($name) = $dbh->selectrow_array($sql_origin);
}
else {
    my $sql_parliament = "SELECT name FROM tp_origin WHERE short_name='Parliament' and lang='$lang'";
    (my $parliament) = $dbh->selectrow_array($sql_parliament);
    my $sql_council = "SELECT name FROM tp_origin WHERE short_name='Council' and lang='$lang'";
    (my $council) = $dbh->selectrow_array($sql_council);
    $name = "$parliament & $council";
}
if ($description =~ /&/o) {
    if ((defined($opts{b}) and $opts{b} == 1) or (defined($opts{a}) and $opts{a} < 4)) {
        $description =~ s/ &.+$//o;
    }
    elsif (defined($opts{b}) or defined($opts{a})) {
        $description =~ s/^.+& //o;
    }
}

# Get translations
my $sql_article = "SELECT translation FROM tp_translations WHERE id='Article' and lang='$lang'";
(my $article) = $dbh->selectrow_array($sql_article);
my $sql_annex = "SELECT translation FROM tp_translations WHERE id='Annex' and lang='$lang'";
(my $annex) = $dbh->selectrow_array($sql_annex);
my $sql_tp = "SELECT translation FROM tp_translations WHERE id='Telecoms Package' and lang='$lang'";
(my $tp) = $dbh->selectrow_array($sql_tp);

# If amended directive is set, print one page with one article or annex by section
if ($opts{a}) {
    # Get title
    my $sql_title = "SELECT number, title, short_name FROM tp_amended WHERE id='$opts{a}' and lang='$lang'";
    my ($number_amended, $title, $short_name) = $dbh->selectrow_array($sql_title);
    $wiki_page .= "<big><big><strong>[http://www.laquadrature.net/lawtracks/telecoms_package/".($lang eq 'en' ? '' : "$lang/")." $tp]: $title ($short_name $number_amended) −&nbsp;$name $description −&nbsp;$date</strong></big></big>\n<hr />\n\n\n";

    # Get chunks
    my $sql = "SELECT nature, number, text FROM tp_text WHERE $where_clause ORDER BY BINARY(number)";
    my $sth = $dbh->prepare($sql);
    $sth->execute;
    my $current_number = '00';
    while (my ($nature, $number, $text) = $sth->fetchrow_array) {
        (my $article_number) = ($number =~ /^([A0-9][0-9](?:@[A-Z])?)/o);
        $article_number =~ s/\@([A-Z])/\l$1/o;

        if ($nature == 3) {
            $article_number =~ s/^A1$/I/o;
            $article_number =~ s/^A2$/II/o;
            $article_number =~ s/^A3$/III/o;
            $article_number =~ s/^A4$/IV/o;
            $article_number =~ s/^A5$/V/o;
            $wiki_page .= "\n\n" unless (($article_number eq $current_number) or $wiki_page =~ /\n\n\n$/o);
            $wiki_page .= "===$annex $article_number===\n\n\n" unless ($article_number eq $current_number);
            $current_number = $article_number;
        }

        if ($article_number ne $current_number) {
            $current_number = $article_number;
            $article_number =~ s/^0//o;
            $wiki_page .= "\n\n" unless $wiki_page =~ /\n\n\n$/o;
            $wiki_page .= "===$article $article_number===\n\n\n";
        }

        if ($nature == 0) {
            $text = "\n'''$text'''";
        }
        if ($nature == 1) {
            $text = "\n''$text''";
        }

        if ($text =~ /^-?\d[\da-zá ]*\. /o) {
            $wiki_page .= "\n";
        }

        $wiki_page .= "$text\n\n";
    }
}
# If only amending directive is set, print one page with one amended directive by section
elsif ($opts{b}) {
    # Get title
    my $sql_title = "SELECT number, title FROM tp_amending WHERE id='$opts{b}' and lang='$lang'";
    my ($number_amending, $title) = $dbh->selectrow_array($sql_title);
    $wiki_page .= "<big><big><strong>[http://www.laquadrature.net/lawtracks/telecoms_package/".($lang eq 'en' ? '' : "$lang/")." $tp]: $title ($number_amending) −&nbsp;$name $description −&nbsp;$date</strong></big></big>\n<hr />\n\n";

    # Get chunks
    my $sql = "SELECT amended, nature, number, text FROM tp_text WHERE $where_clause ORDER BY amended,BINARY(number)";
    my $sth = $dbh->prepare($sql);
    $sth->execute;
    my $current_number = '00';
    my $current_amended = 0;

    while (my ($amended, $nature, $number, $text) = $sth->fetchrow_array) {
        if ($amended != $current_amended) {
            $current_amended = $amended;

            # Get amended title
            my $sql_amended_title = "SELECT number, title, short_name FROM tp_amended WHERE id='$amended' and lang='$lang'";
            my ($number_amended, $title_amended, $short_name) = $dbh->selectrow_array($sql_amended_title);
            $wiki_page .= "\n\n" unless $wiki_page =~ /\n\n\n$/o;
            $wiki_page .= "===$title_amended ($short_name $number_amended)===\n\n\n";
        }

        (my $article_number) = ($number =~ /^([A0-9][0-9](?:@[A-Z])?)/o);
        $article_number =~ s/\@([A-Z])/\l$1/o;

        if ($nature == 3) {
            $article_number =~ s/^A1$/I/o;
            $article_number =~ s/^A2$/II/o;
            $article_number =~ s/^A3$/III/o;
            $article_number =~ s/^A4$/IV/o;
            $article_number =~ s/^A5$/V/o;
            $wiki_page .= "\n\n" unless (($article_number eq $current_number) or $wiki_page =~ /\n\n\n$/o);
            $wiki_page .= "====$annex $article_number====\n\n\n" unless ($article_number eq $current_number);
            $current_number = $article_number;
        }

        if ($article_number ne $current_number) {
            $current_number = $article_number;
            $article_number =~ s/^0//o;
            $wiki_page .= "\n\n" unless $wiki_page =~ /\n\n\n$/o;
            $wiki_page .= "====$article $article_number====\n\n\n";
        }

        if ($nature == 0) {
            $text = "\n'''$text'''";
        }
        if ($nature == 1) {
            $text = "\n''$text''";
        }

        if ($text =~ /^-?\d[\da-zá ]*\. /o) {
            $wiki_page .= "\n";
        }

        $wiki_page .= "$text\n\n";
    }
}
# Otherwise print one page with one amending directive by section (if any)
# and one amended directive by subsection (or section if no amending directive)
else {
    # Get title
    $wiki_page .= "<big><big><strong>[http://www.laquadrature.net/lawtracks/telecoms_package/".($lang eq 'en' ? '' : "$lang/")." $tp]: $name $description −&nbsp;$date</strong></big></big>\n<hr />\n\n";

    # Get chunks
    my $sql = "SELECT amending, amended, nature, number, text FROM tp_text WHERE $where_clause ORDER BY amended,BINARY(number)";
    my $sth = $dbh->prepare($sql);
    $sth->execute;
    my $current_number = '00';
    my $current_amended = 0;
    my $current_amending = 0;

    while (my ($amending, $amended, $nature, $number, $text) = $sth->fetchrow_array) {
        my $subsection = $amending ? '=' : '';

        if ($amending != $current_amending) {
            $current_amending = $amending;

            # Get amended title
            if ($amending) {
                my $sql_amending_title = "SELECT number, title FROM tp_amending WHERE id='$amending' and lang='$lang'";
                my ($number_amending, $title_amending) = $dbh->selectrow_array($sql_amending_title);
                $wiki_page .= "\n\n" unless $wiki_page =~ /\n\n\n$/o;
                $wiki_page .= "<big><strong>$title_amending ($number_amending)</strong></big>\n\n\n";
            }
        }

        if ($amended != $current_amended) {
            $current_amended = $amended;

            # Get amended title
            my $sql_amended_title = "SELECT number, title, short_name FROM tp_amended WHERE id='$amended' and lang='$lang'";
            my ($number_amended, $title_amended, $short_name) = $dbh->selectrow_array($sql_amended_title);
            $wiki_page .= "\n\n" unless $wiki_page =~ /\n\n\n$/o;
            $wiki_page .= "===$subsection$title_amended ($short_name $number_amended)$subsection===\n\n\n";
        }

        (my $article_number) = ($number =~ /^([A0-9][0-9](?:@[A-Z])?)/o);
        $article_number =~ s/\@([A-Z])/\l$1/o;

        if ($nature == 3) {
            $article_number =~ s/^A1$/I/o;
            $article_number =~ s/^A2$/II/o;
            $article_number =~ s/^A3$/III/o;
            $article_number =~ s/^A4$/IV/o;
            $article_number =~ s/^A5$/V/o;
            $wiki_page .= "\n\n" unless (($article_number eq $current_number) or $wiki_page =~ /\n\n\n$/o);
            $wiki_page .= "====$subsection$annex $article_number$subsection====\n\n\n" unless ($article_number eq $current_number);
            $current_number = $article_number;
        }

        if ($article_number ne $current_number) {
            $current_number = $article_number;
            $article_number =~ s/^0//o;
            $wiki_page .= "\n\n" unless $wiki_page =~ /\n\n\n$/o;
            $wiki_page .= "====$subsection$article $article_number$subsection====\n\n\n";
        }

        if ($nature == 0) {
            $text = "\n'''$text'''";
        }
        if ($nature == 1) {
            $text = "\n''$text''";
        }

        if ($text =~ /^\d[\da-z ]*\. /o) {
            $wiki_page .= "\n";
        }

        $wiki_page .= "$text\n\n";
    }
}

# Categories
$tp =~ s/ /_/go;
$wiki_page .= "\n[[Category:$tp]] [[Category:LawTracks".($lang eq 'en' ? '' : "/$lang")."]]\n";

my $file = "$root_dir/mediawiki/$lang/$wiki_name.mediawiki";
mkpath("$root_dir/mediawiki/$lang") unless -d "$root_dir/mediawiki/$lang";
open OUT, '>', $file or die "Cannot open $file: $!\n";
print OUT $wiki_page;
close OUT;
warn "$file done.\n";
