#!/usr/bin/perl

use strict;
use warnings;

use FindBin qw($Bin);
use lib "$Bin";
(my $root_dir = $Bin) =~ s/\/[^\/]*$//o;

use DBI;
use Getopt::Std;
use Term::ReadKey;
use HTML::Diff;
use File::Path;

# Process command line arguments
my %opts;
getopts('sul:a:b:n:p:', \%opts);
my $version_one = shift or die "Usage: $0 [-su] [-l <lang>] [-a <amended_directive>] [-b <amending_directive>] [-n <article_number>] [-p <mysql_password>] <one_version_text> <two_version_text> <three_version_text>\n";
my $version_two = shift or die "Usage: $0 [-su] [-l <lang>] [-a <amended_directive>] [-b <amending_directive>] [-n <article_number>] [-p <mysql_password>] <one_version_text> <two_version_text> <three_version_text>\n";
my $version_three = shift or die "Usage: $0 [-su] [-l <lang>] [-a <amended_directive>] [-b <amending_directive>] [-n <article_number>] [-p <mysql_password>] <one_version_text> <two_version_text> <three_version_text>\n";

# Lang
my $lang = defined($opts{l}) ? $opts{l} : 'en';
my $where_clause = "version_one='$version_one' AND version_two='$version_two' AND version_three='$version_three' AND lang='$lang'";

# Amended
if (defined $opts{a}) {
    die "Amended directive should be:\n* 1 for Access Directive;\n* 2 for Authorisation directive;\n* 3 for Framework Directive;\n* 4 for Universal Service Directive; or\n* 5 for ePrivacy Directive.\n"
        if ($opts{a}<1 or $opts{a}>5);
    $where_clause .= " AND amended='$opts{a}'";
}

# Amending
if (defined $opts{b}) {
    die "Amending directive should be:\n* 1 for COD/2007/0247 (Access, Authorisation and Framework); or\n* 2 for COD/2007/0248 (Universal Service and ePrivacy).\n"
        if ($opts{b}<0 or $opts{b}>2);
    if (defined $opts{a}) {
        die "Amending directive and amended directive are not consistent, amending directive should be:\n* 1 for COD/2007/0247 (Access, Authorisation and Framework); or\n* 2 for COD/2007/0248 (Universal Service and ePrivacy).\n"
        if (($opts{b}==1 and $opts{a}>3) or ($opts{b}==2 and $opts{a}<4));
    }
    $where_clause .= " AND amending='$opts{b}'";
}

# Number
if (defined $opts{n}) {
    die "Article (or recital) number should begin by two (2) digits.\n"
        if ($opts{n} and $opts{n}!~/^\d\d/o);
    die "Amended directive should be set on command line arguments to also set article (or recital, or annex) number.\n"
        unless defined $opts{a};
    $where_clause .= " AND number LIKE '$opts{n}%'";
    $where_clause .= " AND not(number LIKE '$opts{n}\@%'" unless $opts{n} =~ /\@/o;
}

# Connect to database
my ($data_source, $db_user);
if (defined $opts{s}) {
    # ssh -L 6666:127.0.0.1:3306 gibus@pi.serverside.fr
	$data_source = 'dbi:mysql:dbname=pi_tpdb;host=127.0.0.1;port=6666;mysql_enable_utf8=1';
	$db_user = 'pi_tpdb';
}
else {
	$data_source = 'dbi:mysql:dbname=tpdb';
	$db_user = 'gibus';
}

# Get mysql password
my $db_passwd;
if (defined $opts{p}) {
    $db_passwd = $opts{p};
}
else {
    print "Enter mysql passwd for $db_user:\n";
    ReadMode 'noecho';
    $db_passwd = ReadLine 0;
    chomp $db_passwd;
    ReadMode 'normal';
}

my $dbh = DBI->connect($data_source, $db_user, $db_passwd) or die $DBI::errstr;

# Get page wiki name
my $wiki_name = "Telecoms_Package_Diff_";

if ($opts{a}) {
    my $sql_amended_wiki = "SELECT short_name FROM tp_amended WHERE id='$opts{a}' AND lang='en'";
    (my $short_name) = $dbh->selectrow_array($sql_amended_wiki);
    $short_name =~ s/ /_/go;
    $wiki_name .= $short_name;

    if ($opts{n}) {
        $opts{n} =~ s/^0//o;
        if ($opts{n} =~ s/^A(\d)//o) {
            my $roman = $1;
            $roman =~ s/^1$/I/o;
            $roman =~ s/^2$/II/o;
            $roman =~ s/^3$/III/o;
            $roman =~ s/^4$/IV/o;
            $roman =~ s/^5$/V/o;
            $wiki_name .= "_Annexe_$roman";
        }
        elsif ($opts{n} =~ s/^[^:]+://o) {
            $wiki_name .= '_Recital';
        }
        else {
            $wiki_name .= '_Article';
        }
        $opts{n} =~ s/(.)/_$1/go;
        $opts{n} =~ s/_\@_(.)/\l$1/go;
        $wiki_name .= $opts{n};
    }
}
elsif ($opts{b}) {
    my $sql_amending_wiki = "SELECT number FROM tp_amending WHERE id='$opts{b}' AND lang='en'";
    (my $number) = $dbh->selectrow_array($sql_amending_wiki);
    $number =~ s/\//_/go;
    $wiki_name .= $number;
}
else {
    $wiki_name =~ s/_$//o;
}

my $sql_version_one_wiki = "SELECT origin, description FROM tp_versions WHERE id='$version_one' AND lang='en'";
my ($origin_one_wiki, $description_one_wiki) = $dbh->selectrow_array($sql_version_one_wiki);
$wiki_name .= "_$origin_one_wiki" unless $origin_one_wiki eq 'Parliament & Council';
$wiki_name .= "_$description_one_wiki";
my $sql_version_two_wiki = "SELECT origin, description FROM tp_versions WHERE id='$version_two' AND lang='en'";
my ($origin_two_wiki, $description_two_wiki) = $dbh->selectrow_array($sql_version_two_wiki);
$wiki_name .= "_$origin_two_wiki" unless $origin_two_wiki eq 'Parliament & Council';
$wiki_name .= "_$description_two_wiki";
my $sql_version_three_wiki = "SELECT origin, description FROM tp_versions WHERE id='$version_three' AND lang='en'";
my ($origin_three_wiki, $description_three_wiki) = $dbh->selectrow_array($sql_version_three_wiki);
$wiki_name .= "_$origin_three_wiki" unless $origin_three_wiki eq 'Parliament & Council';
$wiki_name .= "_$description_three_wiki";
$wiki_name =~ s/ /_/go;

# Get langs
my $sql_lang = "SELECT id, name FROM tp_lang";
my $langs  = $dbh->selectall_arrayref($sql_lang);

# Start wiki page
my $wiki_page = "__NOEDITSECTION__\n__FORCETOC__\n<div class=\"LanguageLinks\"><table width=\"100%\"><tr valign=\"top\" style=\"background: #EEF3E2\"><td style=\"width: 25px; padding-left: 0.5em;\">[[Image:Geographylogo.png|25px|Languages]]</td><td style=\"padding: 1px 1em 0; background: #F6F9ED;\">";
foreach my $lang (@$langs) {
    $wiki_page .= "'''[[$wiki_name".($lang->[0] eq 'en' ? '' : "/$lang->[0]")."|$lang->[1]]]''' - ";
}
$wiki_page =~ s/ - $//;
$wiki_page .= "</td></tr></table></div>\n\n";

# Get some title info
my $sql_version_one = "SELECT origin, description, date FROM tp_versions WHERE id='$version_one' AND lang='$lang'";
my ($origin_one, $description_one, $date_one) = $dbh->selectrow_array($sql_version_one);
my $name_one;
if ($origin_one ne 'Parliament & Council') {
    my $sql_origin_one = "SELECT name FROM tp_origin WHERE short_name='$origin_one' AND lang='$lang'";
    ($name_one) = $dbh->selectrow_array($sql_origin_one);
}
else {
    my $sql_parliament = "SELECT name FROM tp_origin WHERE short_name='Parliament' AND lang='$lang'";
    (my $parliament) = $dbh->selectrow_array($sql_parliament);
    my $sql_council = "SELECT name FROM tp_origin WHERE short_name='Council' AND lang='$lang'";
    (my $council) = $dbh->selectrow_array($sql_council);
    $name_one = "$parliament & $council";
}
my $sql_version_two = "SELECT origin, description, date FROM tp_versions WHERE id='$version_two' AND lang='$lang'";
my ($origin_two, $description_two, $date_two) = $dbh->selectrow_array($sql_version_two);
my $name_two;
if ($origin_two ne 'Parliament & Council') {
    my $sql_origin_two = "SELECT name FROM tp_origin WHERE short_name='$origin_two' AND lang='$lang'";
    ($name_two) = $dbh->selectrow_array($sql_origin_two);
}
else {
    my $sql_parliament = "SELECT name FROM tp_origin WHERE short_name='Parliament' AND lang='$lang'";
    (my $parliament) = $dbh->selectrow_array($sql_parliament);
    my $sql_council = "SELECT name FROM tp_origin WHERE short_name='Council' AND lang='$lang'";
    (my $council) = $dbh->selectrow_array($sql_council);
    $name_two = "$parliament & $council";
}
my $sql_version_three = "SELECT origin, description, date FROM tp_versions WHERE id='$version_three' AND lang='$lang'";
my ($origin_three, $description_three, $date_three) = $dbh->selectrow_array($sql_version_three);
my $name_three;
if ($origin_three ne 'Parliament & Council') {
    my $sql_origin_three = "SELECT name FROM tp_origin WHERE short_name='$origin_three' AND lang='$lang'";
    ($name_three) = $dbh->selectrow_array($sql_origin_three);
}
else {
    my $sql_parliament = "SELECT name FROM tp_origin WHERE short_name='Parliament' AND lang='$lang'";
    (my $parliament) = $dbh->selectrow_array($sql_parliament);
    my $sql_council = "SELECT name FROM tp_origin WHERE short_name='Council' AND lang='$lang'";
    (my $council) = $dbh->selectrow_array($sql_council);
    $name_three = "$parliament & $council";
}

# Get translations
my $sql_article = "SELECT translation FROM tp_translations WHERE id='Article' and lang='$lang'";
(my $article) = $dbh->selectrow_array($sql_article);
my $sql_annex = "SELECT translation FROM tp_translations WHERE id='Annex' and lang='$lang'";
(my $annex) = $dbh->selectrow_array($sql_annex);
my $sql_tp = "SELECT translation FROM tp_translations WHERE id='Telecoms Package' and lang='$lang'";
(my $tp) = $dbh->selectrow_array($sql_tp);
my $sql_comparison = "SELECT translation FROM tp_translations WHERE id='Comparison' and lang='$lang'";
(my $comparison) = $dbh->selectrow_array($sql_comparison);

# If amended directive is set, print one page with one article or annex by section
if ($opts{a}) {
    # Get title
    my $sql_title = "SELECT number, title, short_name FROM tp_amended WHERE id='$opts{a}' AND lang='$lang'";
    my ($number_amended, $title, $short_name) = $dbh->selectrow_array($sql_title);
    $wiki_page .= "<big><big><strong>[http://www.laquadrature.net/lawtracks/telecoms_package/".($lang eq 'en' ? '' : "$lang/")." $tp]: $comparison $title ($short_name $number_amended)</strong></big></big>\n<hr />\n\n<big><strong>$name_one $description_one / $name_two $description_two / $name_three $description_three −&nbsp;$date_one / $date_two / $date_three</strong></big>\n\n";

    # Get chunks
    my $sql = "SELECT nature, number, text_one, text_two, text_three FROM tp_three_texts WHERE $where_clause ORDER BY BINARY(number)";
    my $sth = $dbh->prepare($sql);
    $sth->execute;
    my $current_number = '00';
    while (my ($nature, $number, $text_one, $text_two, $text_three) = $sth->fetchrow_array) {
        (my $article_number) = ($number =~ /^([A0-9][0-9](?:@[A-Z])?)/o);
        $article_number =~ s/\@([A-Z])/\l$1/o;

        if ($nature == 3) {
            $article_number =~ s/^A1$/I/o;
            $article_number =~ s/^A2$/II/o;
            $article_number =~ s/^A3$/III/o;
            $article_number =~ s/^A4$/IV/o;
            $article_number =~ s/^A5$/V/o;
            if ($article_number ne $current_number) {
                $wiki_page =~ s/\|-\n$/|}\n\n/so;
                $wiki_page .= "===$annex $article_number===\n\n";
                $wiki_page .= "{| border='1' cellpadding='5' cellspacing='0'\n";
                $wiki_page .= "!style='width:33%'| $name_one $description_one !!style='width:33%'| $name_two $description_two !!style='width:33%'| $name_three $description_three\n";
                $wiki_page .= "|-\n";
            }
            $current_number = $article_number;
        }

        if ($article_number ne $current_number) {
            $current_number = $article_number;
            $article_number =~ s/^0//o;
            $wiki_page =~ s/\|-\n$/|}\n\n/so;
            $wiki_page .= "===$article $article_number===\n\n";
            $wiki_page .= "{| border='1' cellpadding='5' cellspacing='0'\n";
            $wiki_page .= "!style='width:33%'| $name_one $description_one !!style='width:33%'| $name_two $description_two !!style='width:33%'| $name_three $description_three\n";
            $wiki_page .= "|-\n";
        }

        if ($nature == 0) {
            $text_one = "'''$text_one'''" if $text_one;
            $text_two = "'''$text_two'''" if $text_two;
            $text_three = "'''$text_three'''" if $text_three;
        }
        if ($nature == 1) {
            $text_one = "''$text_one''" if $text_one;
            $text_two = "''$text_two''" if $text_two;
            $text_three = "''$text_three''" if $text_three;
        }

        $wiki_page .= "|style='border-top:0;border-bottom:0;vertical-align:top'| $text_one ||style='border-top:0;border-bottom:0;vertical-align:top'| $text_two ||style='border-top:0;border-bottom:0;vertical-align:top'| $text_three\n";
        $wiki_page .= "|-\n";
    }
    $wiki_page =~ s/\|-\n$/|}\n\n/so;
}
# If only amending directive is set, print one page with one amended directive by section
elsif ($opts{b}) {
    # Get title
    my $sql_title = "SELECT number, title FROM tp_amending WHERE id='$opts{b}' AND lang='$lang'";
    my ($number_amending, $title) = $dbh->selectrow_array($sql_title);
    $wiki_page .= "<big><big><strong>[http://www.laquadrature.net/lawtracks/telecoms_package/".($lang eq 'en' ? '' : "$lang/")." $tp]: $comparison $title ($number_amending)</strong></big></big>\n<hr />\n\n<big><strong>$name_one $description_one / $name_two $description_two / $name_three $description_three −&nbsp;$date_one / $date_two / $date_three</strong></big>\n\n";

    # Get chunks
    my $sql = "SELECT amended, nature, number, text_one, text_two, text_three FROM tp_three_texts WHERE $where_clause ORDER BY amended,BINARY(number)";
    my $sth = $dbh->prepare($sql);
    $sth->execute;
    my $current_number = '00';
    my $current_amended = 0;

    while (my ($amended, $nature, $number, $text_one, $text_two, $text_three) = $sth->fetchrow_array) {
        if ($amended != $current_amended) {
            $current_amended = $amended;

            # Get amended title
            my $sql_amended_title = "SELECT number, title, short_name FROM tp_amended WHERE id='$amended' AND lang='$lang'";
            my ($number_amended, $title_amended, $short_name) = $dbh->selectrow_array($sql_amended_title);
            $wiki_page =~ s/\|-\n$/|}\n\n/so;
            $wiki_page .= "===$title_amended ($short_name $number_amended)===\n\n";
        }

        (my $article_number) = ($number =~ /^([A0-9][0-9](?:@[A-Z])?)/o);
        $article_number =~ s/\@([A-Z])/\l$1/o;

        if ($nature == 3) {
            $article_number =~ s/^A1$/I/o;
            $article_number =~ s/^A2$/II/o;
            $article_number =~ s/^A3$/III/o;
            $article_number =~ s/^A4$/IV/o;
            $article_number =~ s/^A5$/V/o;
            if ($article_number ne $current_number) {
                $wiki_page =~ s/\|-\n$/|}\n\n/so;
                $wiki_page .= "====$annex $article_number====\n\n";
                $wiki_page .= "{| border='1' cellpadding='5' cellspacing='0'\n";
                $wiki_page .= "!style='width:33%'| $name_one $description_one !!style='width:33%'| $name_two $description_two !!style='width:33%'| $name_three $description_three\n";
                $wiki_page .= "|-\n";
            }
            $current_number = $article_number;
        }

        if ($article_number ne $current_number) {
            $current_number = $article_number;
            $article_number =~ s/^0//o;
            $wiki_page =~ s/\|-\n$/|}\n\n/so;
            $wiki_page .= "====$article $article_number====\n\n";
            $wiki_page .= "{| border='1' cellpadding='5' cellspacing='0'\n";
            $wiki_page .= "!style='width:33%'| $name_one $description_one !!style='width:33%'| $name_two $description_two !!style='width:33%'| $name_three $description_three\n";
            $wiki_page .= "|-\n";
        }

        if ($nature == 0) {
            $text_one = "'''$text_one'''" if $text_one;
            $text_two = "'''$text_two'''" if $text_two;
            $text_three = "'''$text_three'''" if $text_three;
        }
        if ($nature == 1) {
            $text_one = "''$text_one''" if $text_one;
            $text_two = "''$text_two''" if $text_two;
            $text_three = "''$text_three''" if $text_three;
        }

        $wiki_page .= "|style='border-top:0;border-bottom:0;vertical-align:top'| $text_one ||style='border-top:0;border-bottom:0;vertical-align:top'| $text_two ||style='border-top:0;border-bottom:0;vertical-align:top'| $text_three\n";
        $wiki_page .= "|-\n";
    }
    $wiki_page =~ s/\|-\n$/|}\n\n/so;
}
# Otherwise print one page with one amending directive by section (if any)
# and one amended directive by subsection (or section if no amending directive)
else {
    # Get title
    $wiki_page .= "<big><big><strong>[http://www.laquadrature.net/lawtracks/telecoms_package/".($lang eq 'en' ? '' : "$lang/")." $tp]: $comparison $name_one $description_one / $name_two $description_two / $name_three $description_three −&nbsp;$date_one / $date_two / $date_three</strong></big></big>\n<hr />\n\n";

    # Get chunks
    my $sql = "SELECT amending, amended, nature, number, text_one, text_two, text_three FROM tp_three_texts WHERE $where_clause ORDER BY amended,BINARY(number)";
    my $sth = $dbh->prepare($sql);
    $sth->execute;
    my $current_number = '00';
    my $current_amended = 0;
    my $current_amending = 0;

    while (my ($amending, $amended, $nature, $number, $text_one, $text_two, $text_three) = $sth->fetchrow_array) {
        my $subsection = $amending ? '=' : '';

        if ($amending != $current_amending) {
            $current_amending = $amending;

            # Get amended title
            if ($amending) {
                my $sql_amending_title = "SELECT number, title FROM tp_amending WHERE id='$amending' AND lang='$lang'";
                my ($number_amending, $title_amending) = $dbh->selectrow_array($sql_amending_title);
                $wiki_page =~ s/\|-\n$/|}\n\n/so;
                $wiki_page .= "===$title_amending ($number_amending)===\n\n";
            }
        }

        if ($amended != $current_amended) {
            $current_amended = $amended;

            # Get amended title
            my $sql_amended_title = "SELECT number, title, short_name FROM tp_amended WHERE id='$amended' AND lang='$lang'";
            my ($number_amended, $title_amended, $short_name) = $dbh->selectrow_array($sql_amended_title);
            $wiki_page =~ s/\|-\n$/|}\n\n/so;
            $wiki_page .= "===$subsection$title_amended ($short_name $number_amended)$subsection===\n\n";
        }

        (my $article_number) = ($number =~ /^([A0-9][0-9](?:@[A-Z])?)/o);
        $article_number =~ s/\@([A-Z])/\l$1/o;

        if ($nature == 3) {
            $article_number =~ s/^A1$/I/o;
            $article_number =~ s/^A2$/II/o;
            $article_number =~ s/^A3$/III/o;
            $article_number =~ s/^A4$/IV/o;
            $article_number =~ s/^A5$/V/o;
            if ($article_number ne $current_number) {
                $wiki_page =~ s/\|-\n$/|}\n\n/so;
                $wiki_page .= "====$subsection$annex $article_number$subsection====\n\n";
                $wiki_page .= "{| border='1' cellpadding='5' cellspacing='0'\n";
                $wiki_page .= "!style='width:33%'| $name_one $description_one !!style='width:33%'| $name_two $description_two !!style='width:33%'| $name_three $description_three\n";
                $wiki_page .= "|-\n";
            }
            $current_number = $article_number;
        }

        if ($article_number ne $current_number) {
            $current_number = $article_number;
            $article_number =~ s/^0//o;
            $wiki_page =~ s/\|-\n$/|}\n\n/so;
            $wiki_page .= "====$subsection$article $article_number$subsection====\n\n";
            $wiki_page .= "{| border='1' cellpadding='5' cellspacing='0'\n";
            $wiki_page .= "!style='width:33%'| $name_one $description_one !!style='width:33%'| $name_two $description_two !!style='width:33%'| $name_three $description_three\n";
            $wiki_page .= "|-\n";
        }

        if ($nature == 0) {
            $text_one = "'''$text_one'''" if $text_one;
            $text_two = "'''$text_two'''" if $text_two;
            $text_three = "'''$text_three'''" if $text_three;
        }
        if ($nature == 1) {
            $text_one = "''$text_one''" if $text_one;
            $text_two = "''$text_two''" if $text_two;
            $text_three = "''$text_three''" if $text_three;
        }

        $wiki_page .= "|style='border-top:0;border-bottom:0;vertical-align:top'| $text_one ||style='border-top:0;border-bottom:0;vertical-align:top'| $text_two ||style='border-top:0;border-bottom:0;vertical-align:top'| $text_three\n";
        $wiki_page .= "|-\n";
    }
    $wiki_page =~ s/\|-\n$/|}\n\n/so;
}

# Categories
$tp =~ s/ /_/go;
$wiki_page .= "\n[[Category:$tp]] [[Category:LawTracks".($lang eq 'en' ? '' : "/$lang")."]]\n";

my $file = "$root_dir/mediawiki/$lang/$wiki_name.mediawiki";
mkpath("$root_dir/mediawiki/$lang") unless -d "$root_dir/mediawiki/$lang";
open OUT, '>', $file or die "Cannot open $file: $!\n";
print OUT $wiki_page;
close OUT;
warn "$file done.\n";
